@@ -16,6 +16,8 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#define PANGO_ENABLE_BACKEND /* for pango_fc_font_map_cache_clear() */
+
 #include <config.h>
 
 #include <string.h>
@@ -30,6 +32,7 @@
 
 #ifdef GDK_WINDOWING_X11
 #include "x11/gdkx.h"
+#include <pango/pangofc-fontmap.h>
 #endif
 
 #define DEFAULT_TIMEOUT_INITIAL 200
@@ -106,7 +109,10 @@ enum {
   PROP_PRINT_PREVIEW_COMMAND,
   PROP_ENABLE_MNEMONICS,
   PROP_ENABLE_ACCELS,
-  PROP_RECENT_FILES_LIMIT
+  PROP_RECENT_FILES_LIMIT,
+  PROP_IM_MODULE,
+  PROP_RECENT_FILES_MAX_AGE,
+  PROP_FONTCONFIG_TIMESTAMP
 };
 
 
@@ -132,6 +138,7 @@ static void    settings_update_modules           (GtkSettings           *setting
 static void    settings_update_cursor_theme      (GtkSettings           *settings);
 static void    settings_update_resolution        (GtkSettings           *settings);
 static void    settings_update_font_options      (GtkSettings           *settings);
+static gboolean settings_update_fontconfig       (GtkSettings           *settings);
 #endif
 static void    settings_update_color_scheme      (GtkSettings *settings);
 
@@ -570,11 +577,11 @@ gtk_settings_class_init (GtkSettingsClass *class)
    */
   result = settings_install_property_parser (class,
 					     g_param_spec_int ("gtk-tooltip-timeout",
- 							       P_("Tooltip timeout"),
- 							       P_("Timeout before tooltip is shown"),
- 							       0, G_MAXINT,
-							       1500,
- 							       GTK_PARAM_READWRITE),
+							       P_("Tooltip timeout"),
+							       P_("Timeout before tooltip is shown"),
+							       0, G_MAXINT,
+							       500,
+							       GTK_PARAM_READWRITE),
 					     NULL);
 
   g_assert (result == PROP_TOOLTIP_TIMEOUT);
@@ -596,11 +603,11 @@ gtk_settings_class_init (GtkSettingsClass *class)
    */
   result = settings_install_property_parser (class,
 					     g_param_spec_int ("gtk-tooltip-browse-timeout",
- 							       P_("Tooltip browse timeout"),
- 							       P_("Timeout before tooltip is shown when browse mode is enabled"),
- 							       0, G_MAXINT,
-							       100,
- 							       GTK_PARAM_READWRITE),
+							       P_("Tooltip browse timeout"),
+							       P_("Timeout before tooltip is shown when browse mode is enabled"),
+							       0, G_MAXINT,
+							       60,
+							       GTK_PARAM_READWRITE),
 					     NULL);
 
   g_assert (result == PROP_TOOLTIP_BROWSE_TIMEOUT);
@@ -805,6 +812,51 @@ gtk_settings_class_init (GtkSettingsClass *class)
  							       GTK_PARAM_READWRITE),
 					     NULL);
   g_assert (result == PROP_RECENT_FILES_LIMIT);
+
+  /**
+   * GtkSettings:gtk-im-module:
+   *
+   * Which IM module should be used by default.
+   */
+  result = settings_install_property_parser (class,
+					     g_param_spec_string ("gtk-im-module",
+								  P_("Default IM module"),
+								  P_("Which IM module should be used by default"),
+								  NULL,
+								  GTK_PARAM_READWRITE),
+					     NULL);
+  g_assert (result == PROP_IM_MODULE);
+
+  /**
+   * GtkSettings:gtk-recent-files-max-age:
+   *
+   * The maximum age, in days, of the items inside the recently used
+   * resources list. Items older than this setting will be excised
+   * from the list. If set to 0, the list will always be empty; if
+   * set to -1, no item will be removed.
+   *
+   * Since: 2.14
+   */
+  result = settings_install_property_parser (class,
+					     g_param_spec_int ("gtk-recent-files-max-age",
+ 							       P_("Recent Files Max Age"),
+ 							       P_("Maximum age of recently used files, in days"),
+ 							       -1, G_MAXINT,
+							       30,
+ 							       GTK_PARAM_READWRITE),
+					     NULL);
+  g_assert (result == PROP_RECENT_FILES_MAX_AGE);
+
+  result = settings_install_property_parser (class,
+					     g_param_spec_int ("gtk-fontconfig-timestamp",
+ 							       P_("Fontconfig configuration timestamp"),
+ 							       P_("Timestamp of current fontconfig configuration"),
+ 							       G_MININT, G_MAXINT, 0,
+ 							       GTK_PARAM_READWRITE),
+					     NULL);
+  
+  g_assert (result == PROP_FONTCONFIG_TIMESTAMP);
+  
 }
 
 static void
@@ -1018,6 +1070,10 @@ gtk_settings_notify (GObject    *object,
       settings_update_font_options (settings);
       gtk_rc_reset_styles (GTK_SETTINGS (object));
       break;
+    case PROP_FONTCONFIG_TIMESTAMP:
+      if (settings_update_fontconfig (settings))
+	gtk_rc_reset_styles (GTK_SETTINGS (object));
+      break;
     case PROP_CURSOR_THEME_NAME:
     case PROP_CURSOR_THEME_SIZE:
       settings_update_cursor_theme (settings);
@@ -1250,7 +1306,7 @@ free_value (gpointer data)
   
   g_value_unset (&qvalue->public.value);
   g_free (qvalue->public.origin);
-  g_free (qvalue);
+  g_slice_free (GtkSettingsValuePrivate, qvalue);
 }
 
 static void
@@ -1281,7 +1337,7 @@ gtk_settings_set_property_value_internal (GtkSettings            *settings,
   qvalue = g_datalist_id_get_data (&settings->queued_settings, name_quark);
   if (!qvalue)
     {
-      qvalue = g_new0 (GtkSettingsValuePrivate, 1);
+      qvalue = g_slice_new0 (GtkSettingsValuePrivate);
       g_datalist_id_set_data_full (&settings->queued_settings, name_quark, qvalue, free_value);
     }
   else
@@ -1915,6 +1971,43 @@ settings_update_font_options (GtkSettings *settings)
   cairo_font_options_destroy (options);
 }
 
+#ifdef GDK_WINDOWING_X11
+static gboolean
+settings_update_fontconfig (GtkSettings *settings)
+{
+  static guint    last_update_timestamp;
+  static gboolean last_update_needed;
+
+  gint timestamp;
+
+  g_object_get (settings,
+		"gtk-fontconfig-timestamp", &timestamp,
+		NULL);
+
+  /* if timestamp is the same as last_update_timestamp, we already have
+   * updated fontconig on this timestamp (another screen requested it perhaps?),
+   * just return the cached result.*/
+
+  if (timestamp != last_update_timestamp)
+    {
+      PangoFontMap *fontmap = pango_cairo_font_map_get_default ();
+      gboolean update_needed = FALSE;
+
+      if (PANGO_IS_FC_FONT_MAP (fontmap) &&
+	  !FcConfigUptoDate (NULL) && FcInitReinitialize ())
+	{
+	  update_needed = TRUE;
+	  pango_fc_font_map_cache_clear (PANGO_FC_FONT_MAP (fontmap));
+	}
+
+      last_update_timestamp = timestamp;
+      last_update_needed = update_needed;
+    }
+
+  return last_update_needed;
+}
+#endif /* GDK_WINDOWING_X11 */
+
 static void
 settings_update_resolution (GtkSettings *settings)
 {
@@ -1954,7 +2047,7 @@ color_scheme_data_free (ColorSchemeData *data)
       g_free (data->lastentry[i]);
     }
 
-  g_free (data);
+  g_slice_free (ColorSchemeData, data);
 }
 
 static void
@@ -1965,7 +2058,7 @@ settings_update_color_scheme (GtkSettings *settings)
       ColorSchemeData *data;
       GValue value = { 0, };
 
-      data = g_new0 (ColorSchemeData, 1);
+      data = g_slice_new0 (ColorSchemeData);
       data->color_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
 					        (GDestroyNotify) gdk_color_free);
       g_object_set_data_full (G_OBJECT (settings), "gtk-color-scheme",
