@@ -1449,7 +1449,7 @@ build_cache (const gchar *path)
 
   tmp_cache_path = g_build_filename (path, "."CACHE_NAME, NULL);
 
-  if ((fd = open (tmp_cache_path, O_WRONLY | O_CREAT | O_EXCL | O_TRUNC | _O_BINARY, mode)) == -1)
+  if ((fd = g_open (tmp_cache_path, O_WRONLY | O_CREAT | O_EXCL | O_TRUNC | _O_BINARY, mode)) == -1)
     {
       g_printerr (_("Failed to open file %s : %s\n"), tmp_cache_path, g_strerror (errno));
       exit (1);
@@ -1475,7 +1475,6 @@ build_cache (const gchar *path)
       /* Empty table, just close and remove the file */
 
       fclose (cache);
-      close (fd);
       g_unlink (tmp_cache_path);
       exit (0);
     }
@@ -1483,7 +1482,6 @@ build_cache (const gchar *path)
   /* FIXME: Handle failure */
   retval = write_file (cache, files, directories);
   fclose (cache);
-  close (fd);
 
   g_list_foreach (directories, (GFunc)g_free, NULL);
   g_list_free (directories);
