@@ -20,8 +20,6 @@
 
 #include <config.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkstock.h>
-#include <gdk/gdk.h>
 
 gchar *screen2_name = NULL;
 
